<?php

namespace Ilabs\Eclear_Spot;

use Exception;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Abstract_Ilabs_Plugin;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Alerts;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Event_Chain\Interfaces\Wc_Order_Aware_Interface;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Presentation\Form\Interfaces;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Request;
use Isolated\Eclear_Spot\Ilabs\Ilabs_Plugin\Response;
use Ilabs\Eclear_Spot\Remote\Dto\Payload_DTO;
use Ilabs\Eclear_Spot\Remote\Spot_Api_Client;
use Ilabs\Eclear_Spot\Service\Spot_Filesystem;
use Ilabs\Eclear_Spot\Service\Spot_Json_Helper;
use Isolated\Eclear_Spot\Isolated_Guzzlehttp\GuzzleHttp\Client;
use Isolated\Eclear_Spot\Isolated_Guzzlehttp\iLabs\Isolated_Factory;


class Plugin extends Abstract_Ilabs_Plugin {

	/**
	 * @var $limit
	 */
	private static $limit = 60;

	/**
	 * @throws Exception
	 */
	public function enqueue_frontend_scripts() {
		wp_enqueue_style( $this->get_plugin_prefix() . '_front_css',
			$this->get_plugin_css_url() . '/frontend.css'
		);

		wp_enqueue_script( $this->get_plugin_prefix() . '_front_js',
			$this->get_plugin_js_url() . '/front.js',
			[ 'jquery' ],
			1.1,
			true );
	}


	/**
	 * @throws Exception
	 */
	public function enqueue_dashboard_scripts() {
		wp_enqueue_style( $this->get_plugin_prefix() . '_admin_css',
			$this->get_plugin_css_url() . '/admin.css'
		);

		wp_enqueue_script( $this->get_plugin_prefix() . '_admin_js',
			$this->get_plugin_js_url() . '/admin.js',
			[ 'jquery' ],
			1.1,
			true );

		wp_localize_script( $this->get_plugin_prefix() . '_admin_js',

			'eclearSpot',
			[
				'ajaxurl' => set_url_scheme( admin_url( 'admin-ajax.php' ) ),
			]
		);
	}

	public function init() {
		add_filter( 'pre_update_option_eclear_spot_main_keys_secret_key', [ $this, 'init_old_orders_export' ], 10, 2 );
		add_action( 'eclear_spot_export_old_orders', [ $this, 'export_old_orders_callback' ] );

	}

	public function get_guzzlehttp_client(): Client {
		return ( new Isolated_Factory() )->create();
	}

	/**
	 * @throws Exception
	 */
	protected function before_init() {
		(new Hooks())->init();
		$this->implement_general_settings();
		$this->implement_spot();
		$this->ajax();

	}

	private function ajax() {
		add_action( /**
		 * @throws Exception
		 */ 'wp_ajax_eclear_spot_action_test_connection', function () {

			$api = new Spot_Api_Client();

			try {
				$test = $api->test();
				$msg  = $test;
			} catch ( Exception $e ) {
				$msg = $e->getMessage();
			}

			echo wp_json_encode( [ 'status' => 'ok', 'message' => $msg ] );
			die;
		} );
	}


	/**
	 * @throws Exception
	 */
	private function implement_general_settings() {
		$request       = new Request();
		$settings_form = eclear_spot()->get_form_builder();

		$settings_form
			->begin_group( 'main', 'API Settings:' )
			->begin_group( 'keys', 'Keys:', __( 'To obtain API keys register at:',
					'eclear_spot' ) . ' <a target="_blank" href="https://eclear-solutions.com/">https://eclear-solutions.com/</a>' )
			->add_field_text( 'access_key', __( 'Access Key', 'eclear_spot' ) );
		$acces_key_field = $settings_form->get_current_field();
		$settings_form->add_field_text( 'secret_key', __( 'Secret Key', 'eclear_spot' ), '', '', '', true );
		$secret_key_field = $settings_form->get_current_field();
		$settings_form->add_ajax_integration(
			'test_connection',
			__( 'Test connection ', 'eclear_spot' ),
			'' );
		$test_connection_ajax = Interfaces::extract_field_ajax_interface( $settings_form->get_current_field() );
		$settings_form
			->close_group();
		$settings_form = $settings_form->build();

		//var_dump($settings_form);die;

		$settings_feature = eclear_spot()->get_event_chain();
		$settings_feature
			->on_wp_init()
			->action_render_wc_general_settings(
				$settings_form,
				$this->get_from_config('slug').'_general_settings',
				'eClear SPOT'
			)
			->on_wp_ajax( $test_connection_ajax )
			->action( function () use ( $request, $acces_key_field, $secret_key_field ) {
				$acces_key  = $request->get_by_field( $acces_key_field );
				$secret_key = $request->get_by_field( $secret_key_field );
				$client     = new Spot_Api_Client();
				try {
					$client->test();
					$response = new Response( Response::STATUS_OK, __( 'Connection successfully established' ) );
				} catch ( Exception $e ) {
					$response = new Response( Response::STATUS_ERROR,
						__( 'Connection test failed. ' ) . $e->getMessage() );
				}
				$response->send_end_exit();
			} )
			->execute();
	}

	/**
	 * @return void
	 * @throws Exception
	 */
	private function implement_spot() {
		$spot = eclear_spot()->get_event_chain();
		$orders_cache = $spot->get_wp_options_based_cache( 'order_ids' );
		$debug_cache  = $spot->get_wp_options_based_cache( 'debug', 10 );
		$spot
			->on_wc_new_order()
			->action( function ( Wc_Order_Aware_Interface $event ) use ( $orders_cache, $debug_cache ) {
				$orders_cache->push( $event->get_order()->get_id() );
			} )
			->on_wp_cron_every_n_minutes( $this->get_cron_interval(), 'build_json_live' )
			->action( function () use ( $orders_cache, $debug_cache ) {
				$order_ids = $orders_cache->get();
				if ( $order_ids ) {
					try {
						$orders_cache->clear();
						$payload = ( new Payload_DTO() );
						$payload->build( $order_ids );
						$json            = Spot_Json_Helper::create( $payload );
						$spot_filesystem = new Spot_Filesystem();
						$file            = $spot_filesystem->create( $json );
						$upload_response = ( new Spot_Api_Client() )->upload_file( $file );
						$debug_cache->push( $upload_response->get_upload_id() );
					} catch ( Exception $exception ) {
						$debug_cache->push( $exception->getMessage() );
					}
				}
			} )
			->on_wp_init()
			->action( function () use ( $debug_cache ) {
				if ( $debug_cache->get() && defined( 'WOOPLOO_DEBUG' ) ) {

					foreach ( $debug_cache->get() as $k => $message ) {
						( new Alerts() )->add_notice( "Spot: $message" );
					}
				}
			} )
			->execute();
	}

	private function get_cron_interval(): int {
		return defined( 'WOOPLOO_DEBUG' ) ? 1 : 60;
	}

	protected function plugins_loaded_hooks() {
		// TODO: Implement plugins_loaded_hooks() method.
	}


	public function init_old_orders_export( $new_value, $old_value ) {

		$spot        = eclear_spot()->get_event_chain();
		$debug_cache = $spot->get_wp_options_based_cache( 'debug', 10 );
		$debug_cache->clear();

		// probably first time the settings enabled
		if ( empty( $old_value ) && ! empty( $new_value ) ) {

			$client     = new Spot_Api_Client();
			$access_key = isset( $_POST['eclear_spot_main_keys_access_key'] ) ? sanitize_text_field( $_POST['eclear_spot_main_keys_access_key'] ) : null;
			try {
				// if test connection is successful - get all orders ids for this moment
				$client->test( $access_key, $new_value );

				$query      = new \WC_Order_Query( [
					'limit'   => - 1,
					'orderby' => 'date',
					'order'   => 'DESC',
					'return'  => 'ids',
				] );
				$orders_ids = $query->get_orders();

				// if old orders exists - put it to file and create cron task
				if ( ! empty( $orders_ids ) && is_array( $orders_ids ) ) {

					$old_orders_ids_json = wp_json_encode( $orders_ids );
					$spot_filesystem     = new Spot_Filesystem();
					$spot_filesystem->put_old_orders_ids_to_file( $old_orders_ids_json );

					$plugin_prefix = Abstract_Ilabs_Plugin::$initial_instance->get_plugin_prefix();
					if ( ! wp_next_scheduled( $plugin_prefix . '_' . 'export_old_orders' ) ) {
						wp_schedule_event( time(), $plugin_prefix . '_' . 'build_json',
							$plugin_prefix . '_' . 'export_old_orders' );
					}
				}

			} catch ( Exception $e ) {
				error_log( $e->getMessage() );
			}

		}

		return $new_value;

	}


	public function export_old_orders_callback() {

		$end_of_export = false;
		$file          = wp_get_upload_dir()['baseurl'] . '/spot_json/' . 'old_orders.json';

		$request = wp_safe_remote_get( $file );
		if ( 200 === wp_remote_retrieve_response_code( $request ) ) {


			$old_orders_ids = json_decode( wp_remote_retrieve_body( $request ), true );

			if ( is_array( $old_orders_ids ) ) {
				$chunk_to_export = [];
				$rest_of_ids     = [];

				$spot        = eclear_spot()->get_event_chain();
				$debug_cache = $spot->get_wp_options_based_cache( 'debug', 10 );

				// divide list of orders ids if its count exceed limit
				if ( count( $old_orders_ids ) > self::$limit ) {
					$chunk_to_export = array_slice( $old_orders_ids, 0, self::$limit );
					$rest_of_ids     = array_slice( $old_orders_ids, self::$limit, count( $old_orders_ids ) );
				} else {
					$chunk_to_export = $old_orders_ids;
					$end_of_export   = true;
				}

				$payload = new Payload_DTO();
				$payload->build( $chunk_to_export );
				$json            = Spot_Json_Helper::create( $payload );
				$spot_filesystem = new Spot_Filesystem();
				$export          = $spot_filesystem->create( $json );

				try {

					$upload_response = ( new Spot_Api_Client() )->upload_file( $export );

					if ( ! $end_of_export ) {
						// put new file with rest of ids to export
						$rest_of_ids_json = wp_json_encode( $rest_of_ids );
						$spot_filesystem  = new Spot_Filesystem();
						$spot_filesystem->put_old_orders_ids_to_file( $rest_of_ids_json );

						$debug_cache->push( 'exporting old orders - ' . self::$limit . ' from ' . count( $old_orders_ids ) );
					} else {
						// clear cron task
						$plugin_prefix = Abstract_Ilabs_Plugin::$initial_instance->get_plugin_prefix();
						wp_clear_scheduled_hook( $plugin_prefix . '_' . 'export_old_orders' );
						$debug_cache->push( 'old orders export finished' );
						// we need abs path to delete file
						$uploads_dir      = eclear_spot()->get_uploads_dir();
						$target_directory = $uploads_dir . DIRECTORY_SEPARATOR . 'spot_json';
						$file_ids         = $target_directory . DIRECTORY_SEPARATOR . 'old_orders.json';
						wp_delete_file( $file_ids );
					}

				} catch ( Exception $exception ) {
					//error_log( $e->getMessage() );
				}
			}
		}
	}

}
